"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.ValidateRequestAsync = exports.ValidateRequest = exports.failValidationForField = exports.validateRequestAsync = exports.validateRequest = exports.resolveValidationError = exports.getValidatedFieldName = void 0;
const joi_1 = __importDefault(require("joi"));
const types_1 = require("../types");
const response_1 = require("./response");
const error_1 = require("./error");
const getValidatedFieldName = (options) => {
    if (Array.isArray(options.state.path)) {
        return options.state.path.join('.');
    }
    else if (typeof options.state.path === 'string') {
        return options.state.path;
    }
    else {
        return ``;
    }
};
exports.getValidatedFieldName = getValidatedFieldName;
const resolveValidationError = (valError) => {
    if (!valError) {
        return new types_1.ValidationError({});
    }
    let errors = {};
    valError.details.map((error) => {
        errors[error.path.join('.')] = error.message;
    });
    return new types_1.ValidationError(errors);
};
exports.resolveValidationError = resolveValidationError;
const validateRequest = (schema, data, options = {
    abortEarly: false
}) => {
    const valResult = schema.validate(data, options);
    if (valResult.error) {
        throw (0, exports.resolveValidationError)(valResult.error);
    }
};
exports.validateRequest = validateRequest;
const validateRequestAsync = async (schema, data, options = {
    abortEarly: false
}) => {
    /**
     * Joi will throw built-in ValidationError for async validation
     */
    await schema.validateAsync(data, options);
};
exports.validateRequestAsync = validateRequestAsync;
const failValidationForField = (field, errorMessage) => {
    throw new types_1.ValidationError({
        [field]: errorMessage
    });
};
exports.failValidationForField = failValidationForField;
function ValidateRequest(schemaOrValidator) {
    return (target, propertyKey, descriptor) => {
        const originalMethod = descriptor.value;
        descriptor.value = function (req, res, next) {
            try {
                (0, exports.validateRequest)((0, types_1.isRequestValidator)(schemaOrValidator)
                    ? schemaOrValidator.getSchema(req, res)
                    : schemaOrValidator, req.body);
            }
            catch (e) {
                if (e instanceof types_1.ValidationError) {
                    return (0, response_1.validationErrorResponse)(res, e);
                }
                return (0, error_1.invokeRequestErrorHandler)(e, req, res);
            }
            return originalMethod.apply(this, [req, res, next]);
        };
        return descriptor;
    };
}
exports.ValidateRequest = ValidateRequest;
const ValidateRequestAsync = (schemaOrValidator) => {
    return function (scope, methodName, descriptor) {
        const originalMethod = descriptor.value;
        descriptor.value = async function (req, res, ...args) {
            try {
                await (0, exports.validateRequestAsync)((0, types_1.isRequestValidator)(schemaOrValidator)
                    ? schemaOrValidator.getSchema(req, res)
                    : schemaOrValidator, req.body);
            }
            catch (e) {
                if (e instanceof types_1.ValidationError) {
                    return (0, response_1.validationErrorResponse)(res, e);
                }
                else if (e instanceof joi_1.default.ValidationError) {
                    return (0, response_1.validationErrorResponse)(res, (0, exports.resolveValidationError)(e));
                }
                return (0, error_1.invokeRequestErrorHandler)(e, req, res);
            }
            return originalMethod.apply(this, [req, res, ...args]);
        };
    };
};
exports.ValidateRequestAsync = ValidateRequestAsync;
